#include "application.h"
#include "consoleapplication.h"
#include "guiapplication.h"

#include "utils.h"

#include <iostream>

using namespace std;

// PARSE ARGUMENTS
enum argument{
    modeArgument,
    inputArgument,
    orientationArgument,
    disparityArgument,
    threadsArgument,
    outputArgument,
    parametersArgument,
    sequenceArgument,
    helpArgument,
    unknownArgument
};

static argument getArgument(string argument){

    if (!argument.compare("-m")) return modeArgument;
    if (!argument.compare("-i")) return inputArgument;
    if (!argument.compare("-o")) return orientationArgument;
    if (!argument.compare("-d")) return disparityArgument;
    if (!argument.compare("-t")) return threadsArgument;
    if (!argument.compare("-r")) return outputArgument;
    if (!argument.compare("-p")) return parametersArgument;
    if (!argument.compare("-s")) return sequenceArgument;

    if (!argument.compare("-h")) return helpArgument;

    return unknownArgument;
}

int main(int argc, char *argv[]){

    string mode = "GUI"; // DEFAULT GUI
    string input[2];
    string orientation;
    string disparity;
    string threads;
    string output;
    string parameters;
    string sequence;

    for (int i = 0; i < argc; i++){
        switch(getArgument(argv[i])){
            case modeArgument:{
                mode = argv[i+1];
                break;
            }
            case inputArgument:{
                input[0] = argv[i+1];
                if(Utils::isFile(argv[i+2])) input[1] = argv[i+2];
                else input[1] = input[0];
                break;
            }
            case orientationArgument:{
                orientation = argv[i+1];
                break;
            }
            case disparityArgument:{
                disparity = argv[i+1];
                break;
            }
            case threadsArgument:{
                threads = argv[i+1];
                break;
            }
            case outputArgument:{
                output = argv[i+1];
                break;
            }
            case parametersArgument:{
                parameters = argv[i+1];
                break;
            }
            case sequenceArgument:{
                sequence = argv[i+1];
                break;
            }
            case helpArgument:{
                cout << "USAGE: DisparityChecker.exe [ARGUMENTS]"  << endl;
                cout << endl;

                cout << "ARGUMENTS:"  << endl;
                string m("-m GUI | CONSOLE"); m.resize(35, ' ');
                cout << "\t" << m <<  "-- application mode, default GUI"  << endl;
                string i("-i [PATH]"); i.resize(35, ' ');
                cout << "\t" << i << "-- input files path, required"  << endl;
                string o("-o VERTICALLY | HORIZONTALLY"); o.resize(35, ' ');
                cout << "\t" << o << "-- frame orientation, required (at least one input file)"  << endl;
                string d("-q [DISPARITY METHOD]"); d.resize(35, ' ');
                cout << "\t" << d << "-- disparity method, required"  << endl;
                string t("-t [THREADS AMOUNT]"); t.resize(35, ' ');
                cout << "\t" << t << "-- calculation threads, required"  << endl;
                string r("-r [PATH]"); r.resize(35, ' ');
                cout << "\t" <<  r << "-- output path, required"  << endl;
                string p("-p [DISPARITY METHOD PARAMETERS]"); p.resize(35, ' ');
                cout << "\t" << p << "-- parameters, optional"  << endl;
                string s("-s [SEQUENCE]"); s.resize(35, ' ');
                cout << "\t" << s << "-- frames sequence, optional"  << endl;
                string h("-h"); h.resize(35, ' ');
                cout << "\t" << h << "-- console help"  << endl;
                cout << endl;

                cout << "EXAMPLE:"  << endl;
                cout << "\t" << "DisparityChecker.exe -m GUI"  << endl;
                cout << "\t" << "DisparityChecker.exe -m CONSOLE -f ~/movie.mov -o HORIZONTALLY -q Measure -t 8 -r ~/results"  << endl;
                cout << endl;

                return 0;
                break;
            }
            default:
                break;
        }
    }

    //RUN APPLICATION
    Application* application=NULL;

    //RUN GUI APPLICATION
    if(!mode.compare("GUI")){
        if ((((!input[0].empty() && Utils::isFile(input[0]) && !orientation.empty()) || (!input[0].empty() && !input[1].empty() && input[0].compare(input[1]) && Utils::isFile(input[0]) && Utils::isFile(input[1]))) && !disparity.empty() && !threads.empty() && !output.empty())){
            application = new GUIApplication();
            return application->execute(input, orientation, disparity, threads, output, parameters, sequence);
        }else{
            cerr << "Wrong application arguments for GUI mode" << endl;
            cerr << "HELP: DisparityChecker -h" << endl;
            cerr << endl;
            return -1;
        }
    }
    //RUN CONSOLE APPLICATION
    if(!mode.compare("CONSOLE")){
        if ((((!input[0].empty() && Utils::isFile(input[0]) && !orientation.empty()) || (!input[0].empty() && !input[1].empty() && input[0].compare(input[1]) && Utils::isFile(input[0]) && Utils::isFile(input[1]))) && !disparity.empty() && !threads.empty() && !output.empty())){
            application = new ConsoleApplication();
            return application->execute(input, orientation, disparity, threads, output, parameters, sequence);
        }else{
            cerr << "Wrong application arguments for CONSOLE mode" << endl;
            cerr << "HELP: DisparityChecker -h" << endl;
            cerr << endl;
            return -1;
        }
    }

    //WRONG APPLICATION MODE
    if((mode.compare("GUI") && mode.compare("CONSOLE"))){
        cerr << "Wrong application MODE" << endl;
        cerr << "HELP: DisparityChecker -h" << endl;
        cerr << endl;
        return -1;
    }

    return 0;
}
