#include "r3dutils.h"

#include <utils.h>
#include <fstream>

unsigned char* R3DUtils::allocate(unsigned int & sizeNeeded){
    unsigned char * buffer = (unsigned char *)malloc(sizeNeeded + 15U);
    if (!buffer) return NULL;
    sizeNeeded = 0U;
    uintptr_t ptr = (uintptr_t)buffer;
    if ((ptr % 16U) == 0U) return buffer;
    sizeNeeded = 16U - (ptr % 16U);

    return buffer + sizeNeeded;
}

Mat R3DUtils::toMat(unsigned char* buffer, int width, int height){

    vector<Mat> channels;
    for (int i = 4; i >= 0; i-=2) {
        channels.push_back(Mat(height, width, CV_16UC1, buffer + (i*(width*height))));
    }

    Mat mat;
    merge(channels, mat);

    return mat;
}

bool R3DUtils::isVerticalFlip(string path){
    string filename(path);

    if(filename.find_last_of(Settings::getInstance()->DOT_STRING) != string::npos){
        string rmd = filename.replace(filename.find_last_of(Settings::getInstance()->DOT_STRING)+1, 3, "RMD");
        if(Utils::isFile(rmd)){
            ifstream file(rmd.c_str());
            string line;

            while(getline(file, line)){
                 if(line.find("<flipvertical type=\"bool\" value=\"true\"\/>") != string::npos){
                    return true;
                 }
            }
        }
    }

    return false;
}

bool R3DUtils::isHorizontalFlip(string path){
    string filename(path);

    if(filename.find_last_of(Settings::getInstance()->DOT_STRING) != string::npos){
        string rmd = filename.replace(filename.find_last_of(Settings::getInstance()->DOT_STRING)+1, 3, "RMD");
        if(Utils::isFile(rmd)){
            ifstream file(rmd.c_str());
            string line;

            while(getline(file, line)){
                 if(line.find("<fliphorizontal type=\"bool\" value=\"true\"\/>") != string::npos){
                    return true;
                 }
            }
        }
    }

    return false;
}
