#include <fstream>

#include "serialize.h"
#include "utils.h"
#include "direct.h"

#include <iostream>

#include <iomanip>


using namespace std;

void Serialize::prepare(string output){
    int position = 0;
    while(position != std::string::npos)
    {
        position = output.find(Settings::getInstance()->SLASH_STRING, position +1);
        if(position != std::string::npos){
            string path = output.substr(0, position+1);
            mkdir(path.c_str());
        }
    }
}

void Serialize::multi(Mat left, Mat right, int id, int size, vector<Result*> results, Mat summary, string name, string output){
    Serialize::prepare(output);

    imwrite(output + Utils::toString(id) + Settings::getInstance()->LINE_STRING + Utils::toString(size) + Settings::getInstance()->LINE_STRING + string("LEFT") +  Settings::getInstance()->DOT_STRING + string("png"), left);
    imwrite(output + Utils::toString(id) + Settings::getInstance()->LINE_STRING + Utils::toString(size) + Settings::getInstance()->LINE_STRING + string("RIGHT") +  Settings::getInstance()->DOT_STRING + string("png"), right);

    Serialize::plain(id, size, results, name, output);
    Serialize::images(id, size, results, name, output);

    imwrite(output + Utils::toString(id) + Settings::getInstance()->LINE_STRING + Utils::toString(size) + Settings::getInstance()->LINE_STRING + string("SUMMARY") +  Settings::getInstance()->DOT_STRING + string("png"), summary);

}

void Serialize::single(Mat left, Mat right, int id, int size, vector<Result*> results, string name, string output){
    Serialize::csv(id, size, results, name, output);
    //Serialize::table(left, right, id, size, results, name, output);
}

void Serialize::plain(int id, int size, vector<Result*> results, string name, string output){
    Serialize::prepare(output);

    string filename = output + Utils::toString(id) + Settings::getInstance()->LINE_STRING + Utils::toString(size) + Settings::getInstance()->LINE_STRING + string("RESULT") +  Settings::getInstance()->DOT_STRING + string("txt");

    ofstream file;
    file.open (filename, ofstream::out);

    for(Result* result: results) {
        for(Result::KeyValue* data: result->getData()){
            file << result->getName() << Settings::getInstance()->SPACE_STRING << data->getKey() << Settings::getInstance()->SPACE_STRING << Settings::getInstance()->COLON_STRING << Settings::getInstance()->SPACE_STRING << data->getValue() << endl;
        }
    }
    file.close();
}

void Serialize::images(int id, int size, vector<Result*> results, string name, string output){
    Serialize::prepare(output);

    for(Result* result: results) {
        for(Result::KeyImage* image: result->getImages()){
            imwrite(output + Utils::toString(id) + Settings::getInstance()->LINE_STRING + Utils::toString(size) + Settings::getInstance()->LINE_STRING + result->getName() + Settings::getInstance()->LINE_STRING + image->getKey() +  Settings::getInstance()->DOT_STRING + string("png"), image->getImage());
        }
    }
}

void Serialize::csv(int id, int size, vector<Result*> results, string name, string output){
    Serialize::prepare(output);

    string filename = output + string("RESULTS") +  Settings::getInstance()->DOT_STRING + string("csv");

    for(Result* result: results){
        if(Utils::isEmpty(filename)){
            ofstream file;
            file.open (filename, ofstream::app);
            file << Settings::getInstance()->COMA_STRING << "ID" << Settings::getInstance()->COMA_STRING << "SIZE" << Settings::getInstance()->COMA_STRING;
                for (unsigned int i = 0; i < result->getData().size(); i++) {
                    file << result->getData().at(i)->getKey();
                    if(i!= result->getData().size()-1){
                        file << Settings::getInstance()->COMA_STRING;
                    }else{
                        file << endl;
                    }
                }
            file.close();
        }

        ofstream file;
        file.open (filename, ofstream::app);
        file << result->getName() << Settings::getInstance()->COMA_STRING << id << Settings::getInstance()->COMA_STRING << size << Settings::getInstance()->COMA_STRING;
        for (unsigned int i = 0; i < result->getData().size(); i++) {
            file << result->getData().at(i)->getValue();
            if(i!= result->getData().size()-1){
                file << Settings::getInstance()->COMA_STRING;
            }else{
                file << endl;
            }
        }
        file.close();
    }
}

string format(double value){
    stringstream stream;
    stream << fixed << setprecision(3) << roundf(value*1000)/1000;
    string text = stream.str();
    return text;
}
void Serialize::table(Mat left, Mat right, int id, int size, vector<Result*> results, string name, string output){
    Serialize::prepare(output);

    string filename = output + string("TABLE") +  Settings::getInstance()->DOT_STRING + string("csv");

    if(Utils::isEmpty(filename)){
        ofstream file;
        file.open (filename, ofstream::app);
        file << "Nazwa,Rozdzielczość,Klatka,Dysparycja pionowa - średnia [px],Dysparycja pionowa - średnia [%],Dysparycja pionowa - mediana [px],Dysparycja pionowa - mediana [%],Błąd rotacji - średnia [deg],Błąd rotacji - mediana [deg],Bład skali - średnia,Błąd skali - mediana" << endl;
        file.close();
    }

    ofstream file;
    file.open (filename, ofstream::app);
    file <<  (name.find("_L") != string::npos ? name.replace(name.find("_L"), 2, "") : name.replace(name.find("-L"), 2, "")) << "," << left.cols << ":" << left.rows << "," << id << ",";

    for(Result* result: results){
        if(!result->getName().compare("VERTICAL")){
            for (unsigned int i = 0; i < result->getData().size(); i++) {
                if(!result->getData().at(i)->getKey().compare("AVERAGE")){
                    file << format(result->getData().at(i)->getValue()) << "," << format(result->getData().at(i)->getValue() * 100 /left.rows) << ",";
                }

                if(!result->getData().at(i)->getKey().compare("MEDIAN")){
                    file << format(result->getData().at(i)->getValue()) << "," << format(result->getData().at(i)->getValue() * 100 /left.rows) << ",";
                }
            }
        }

        if(!result->getName().compare("ROTATION")){
            for (unsigned int i = 0; i < result->getData().size(); i++) {
                if(!result->getData().at(i)->getKey().compare("AVERAGE")){
                    file << format(result->getData().at(i)->getValue()) << ",";
                }

                if(!result->getData().at(i)->getKey().compare("MEDIAN")){
                    file << format(result->getData().at(i)->getValue()) << ",";
                }
            }
        }

        if(!result->getName().compare("SCALE")){
            for (unsigned int i = 0; i < result->getData().size(); i++) {
                if(!result->getData().at(i)->getKey().compare("AVERAGE")){
                    file << format(result->getData().at(i)->getValue()) << ",";
                }

                if(!result->getData().at(i)->getKey().compare("MEDIAN")){
                    file << format(result->getData().at(i)->getValue()) << endl;
                }
            }
        }

    }

    file.close();
}
