#include "statistic.h"

#include <opencv2/highgui/highgui.hpp>

using namespace cv;

double Statistic::points(vector<Disparity*> values){
    return values.size();
}

double Statistic::average(vector<Disparity*> values){
    if(values.size()==0) return values.size();

    double average=0;
    for(Disparity* value: values){
        average+=value->getValue();
    }
    average/=values.size();

    return average;
}

double Statistic::averageSD(vector<Disparity*> values){
    if(values.size()==0) return values.size();

    double average = Statistic::average(values);
    double variance = 0;
    for(Disparity* value: values){
        variance+=((value->getValue()-average) * (value->getValue()-average));
    }
    variance/=values.size();
    return sqrt(variance);
}

double Statistic::averageABS(vector<Disparity*> values){
    if(values.size()==0) return values.size();

    double average=0;
    for(Disparity* value: values){
        average+=abs(value->getValue());
    }
    average/=values.size();

    return average;
}

double Statistic::median(vector<Disparity*> values){
    if(values.size()==0) return values.size();

    sort(values.begin(), values.end(), Disparity::comparator);

    double median;
    if (values.size()%2 == 0){
        unsigned int index1 = (values.size()/2)-1;
        unsigned int index2 = (values.size()/2);
        median = (values.at(index1)->getValue() + values.at(index2)->getValue())/2;
    }else{
        unsigned int index = values.size()/2;
        median = values.at(index)->getValue();
    }
    return median;
}

double Statistic::medianAD(vector<Disparity*> values){
    if(values.size()==0) return values.size();

    double median = Statistic::median(values);

    vector<double> absoluteDifferences;
    for(Disparity* value: values){
        absoluteDifferences.push_back(abs(value->getValue()-median));
    }
    sort(absoluteDifferences.begin(), absoluteDifferences.end());

    double medianAD;
    if (absoluteDifferences.size()%2 == 0){
        unsigned int index1 = (absoluteDifferences.size()/2)-1;
        unsigned int index2 = (absoluteDifferences.size()/2);
        medianAD = (absoluteDifferences.at(index1) + absoluteDifferences.at(index2))/2;
    }else{
        unsigned int index = absoluteDifferences.size()/2;
        medianAD = absoluteDifferences.at(index);
    }
    return medianAD;
}


double Statistic::max(vector<Disparity*> values){
    if(values.size()==0) return values.size();
    double max= DBL_MIN;

    for (Disparity* value: values){
        if(abs(value->getValue()) > max){
            max = abs(value->getValue());
        }
    }
    return max;
}
