#include "thread.h"
#include "serialize.h"

Thread::Thread(Application* application, DisparityCheckerInterface* disparityChecker, Mat left, Mat right, int id, int size, string name, string output){
    this->application = application;
    this->disparityChecker = disparityChecker;
    left.copyTo(this->left);
    right.copyTo(this->right);
    this->id = id;
    this->size = size;
    this->name = name;
    this->output = output;
}

Thread::~Thread(){
    this->left.release();
    this->right.release();
    for(unsigned int i = 0; i < this->results.size(); i++){
        this->results.at(i)->~Result();
    }
    this->summary.release();
}

void Thread::run(){
    this->results = this->disparityChecker->execute(nullptr, this->left, this->right);
    this->summary = this->disparityChecker->summary(this->results);
    Serialize::multi(this->left, this->right, this->id, this->size, this->results, this->summary, this->name, this->output);
}

void Thread::call(){
    Serialize::single(this->left, this->right, this->id, this->size, this->results, this->name, this->output);
    this->application->update(this->left, this->right, this->id, this->size, this->results, this->summary, this->name);
}
