#include "utils.h"

#include <fstream>

bool Utils::isFile(string input){
    ifstream file(input);

    return file.good();
}

bool Utils::isEmpty(string input){
    ifstream file;
    file.open(input, ifstream::app);
    bool empty = file.peek() == std::ifstream::traits_type::eof();
    file.close();

    return empty;
}

string Utils::getFilename(string input){
    string path(input);

    string fullname = path.substr(path.find_last_of(Settings::getInstance()->SLASH_STRING) + 1);

    return fullname.substr(0, fullname.find_last_of(Settings::getInstance()->DOT_STRING));
}

bool Utils::isExtension(string extension, vector<string> extensions){
    for(unsigned int i=0; i<extensions.size(); i++){
        if (!extension.compare(extensions.at(i))) return true;
    }

    return false;
}

string Utils::getExtension(string input){
    string path(input);

    if(path.find_last_of(Settings::getInstance()->DOT_STRING) != string::npos){
        string extension = path.substr(path.find_last_of(Settings::getInstance()->DOT_STRING)+1);
        for (auto & iterator: extension) iterator = toupper(iterator);
        return extension;
    }

    return Settings::getInstance()->EMPTY_STRING;
}

Utils::Format Utils::getFormat(string input[]){
    if(!Utils::getExtension(input[0]).compare(Utils::getExtension(input[1]))){
        if(Utils::isExtension(Utils::getExtension(input[0]), Settings::getInstance()->NATIVE_EXTENSIONS)) return Format::NATIVE;
        if(Utils::isExtension(Utils::getExtension(input[0]), Settings::getInstance()->MOVIE_EXTENSIONS)) return Format::MOVIE;
        if(Utils::isExtension(Utils::getExtension(input[0]), Settings::getInstance()->IMAGE_EXTENSIONS)) return Format::IMAGE;
    }

    return Format::UNKNOWN;
}

vector<int> Utils::getSequences(string sequence){
    string seq(sequence);

    vector<int> sequences;

    if(seq.find(Settings::getInstance()->DASH_STRING) != string::npos){
        int index =  seq.find(Settings::getInstance()->DASH_STRING);
        string start = seq.substr(0, index);
        string end = seq.substr(index+1, seq.length());
        for(int i = atoi(start.c_str()); i<=atoi(end.c_str()); i++){
            sequences.push_back(i);
        }
        return sequences;
    }

    if(seq.find(Settings::getInstance()->COMA_STRING) != string::npos){
        seq += Settings::getInstance()->COMA_STRING;
        int index;
        while ((index = seq.find(Settings::getInstance()->COMA_STRING)) != string::npos) {
            string value = seq.substr(0, index);
            sequences.push_back(atoi(value.c_str()));
            seq.erase(0, index+1);
        }
        return sequences;
    }

    return sequences;
}

string Utils::toString(int i){
    string value;
    if(i < 0){
        value = "-";
        i = -i;
    }

    if(i > 9) value += toString(i / 10);

    value += i % 10 + 48;

    return value;
}
