#include "visualize.h"

#include <iomanip>

#define RADIUS 50

Mat Visualize::circles(Mat image, vector<Disparity*> disparities){
    double max = Statistic::max(disparities);

    Mat circles; image.copyTo(circles);
    for(Disparity* disparity: disparities){
        if(abs(disparity->getValue()) > abs(Statistic::median(disparities))){
            int radius = abs(RADIUS*disparity->getValue()/max);
            if(radius>0){
                if(disparity->getValue()>=0){
                    circle(circles, disparity->getPoint(), radius, CV_RGB(255,0,0), -1);
                }else{
                    circle(circles, disparity->getPoint(), radius, CV_RGB(0,0,255), -1);
                }
                stringstream stream;
                stream << fixed << setprecision(2) << disparity->getValue();
                string text = stream.str();
                int type = 8;
                Size size = getTextSize(text, FONT_HERSHEY_DUPLEX, 1, 1, &type);
                Point position = Point(disparity->getPoint().x-(size.width/2), disparity->getPoint().y+(size.height/2));
                putText(circles, text, position, FONT_HERSHEY_DUPLEX, 1, CV_RGB(0,0,0), 1, type, false);
            }
        }
    }

    return circles;
}
Mat Visualize::matches(Mat left, Mat right, Correlation* correlation){
    Mat matches;

    drawMatches(left, correlation->getKeypointsLeft(), right, correlation->getKeypointsRight(), correlation->getMatches(), matches);
    return matches;
}

Mat Visualize::anaglyph(Mat left, Mat right){
    Mat leftChannels[3];        Mat rightChannels[3];
    split(left,leftChannels);   split(right,rightChannels);

    vector<Mat> channels = {rightChannels[0], rightChannels[1], leftChannels[2]};
    Mat anaglyph;
    merge(channels, anaglyph);

    return anaglyph;
}
